class Block < ActiveRecord::Base
  belongs_to :block_version
  validates :block_version, presence: true

  attr_translatable :text

  scope :by_key, -> (key) { where(key: key) }
  scope :active, -> { joins(:block_version).where('block_versions.active' => true) }

  def self.last(key, default_text = nil)
    block = Block.active.by_key(key).last
    block = Block.new(key: key, text: default_text) if block.nil?
    block.prepare
  end

  def self.last_for_admin(key, admin, default_text = nil)
    block = Block.by_key(key).joins(:block_version).
        where('block_versions.active=? or block_versions.account_id=?', true, admin).last

    if block.nil?
      block_version = BlockVersion.where(account: admin).last
      if block_version.nil? or block_version.active?
        block_version = BlockVersion.create(account: admin, active: false)
      end
      block = Block.new(key: key, block_version: block_version, text: default_text)
    end

    block.prepare
  end

  def prepare
    @text = text.present? ? text : key.to_s.humanize
    @tag = :div
    @tag_options = []
    @tag_class = "block block_key_#{key}"
    self
  end

  def apply_options(options)
    if options.is_a? Hash
      options.each_pair do |key, value|
        if key.to_sym == :class
          apply_class_options value
        elsif key.to_sym == :tag
          @tag = value.to_sym
        else
          @tag_options.append "#{key}='#{value}'"
        end
      end
    else
      apply_class_options options
    end
    self
  end

  def apply_class_options(options)
    if options.is_a? Array
      @tag_class += ' ' + options.join(' ')
    elsif options.present?
      @tag_class += ' ' + options.to_s
    end
  end

  def html
    @tag_options.append "class='#{@tag_class}'"
    @tag_options.append 'data-container' if tag_container?

    "<#{@tag} #{@tag_options.join(' ')}>#{tag_container? ? html_text : plain_text}</#{@tag}>".html_safe
  end

  def html_text
    (text || @text).html_safe
  end

  def plain_text
    ActionView::Base.full_sanitizer.sanitize(text || @text)
  end

  private
  def tag_container?
    @tag != :p && @tag != :span
  end

end
